@@ -1,5 +1,5 @@
 #' @title Coupled leaf gas exchange model
-#' @description A coupled photosynthesis - stomatal conductance model, based on the Farquhar model of photosynthesis, and a Ball-Berry type model of stomatatal conductance. Includes temperature sensitivity of photosynthetic parameters, dark respiration (optionally calculated from leaf temperature), and mesophyll conductance. 
+#' @description A coupled photosynthesis - stomatal conductance model, based on the Farquhar model of photosynthesis, and a Ball-Berry type model of stomatatal conductance. Includes options for temperature sensitivity of photosynthetic parameters, dark respiration (optionally calculated from leaf temperature), and mesophyll conductance. 
 #' @param VPD Vapour pressure deficit (kPa)
 #' @param Ca Atmospheric CO2 concentration (ppm)
 #' @param PPFD Photosynthetic photon flux density ('PAR') (mu mol m-2 s-1)
@@ -17,13 +17,14 @@
 #' @param Jmax Maximum rate of electron transport at 25 degrees C (mu mol m-2 s-1)
 #' @param Vcmax Maximum carboxylation rate at 25 degrees C (mu mol m-2 s-1)
 #' @param gmeso Mesophyll conductance (mol m-2 s-1). If not NULL (the default), Vcmax and Jmax are chloroplastic rates.
-#' @param Rd Dark respiration rate (mu mol m-2 s-1), optional (if not provided, calculated from Tleaf)
+#' @param Rd Dark respiration rate (mu mol m-2 s-1), optional (if not provided, calculated from Tleaf, Rd0, Q10 and TrefR)
 #' @param Rd0 Dark respiration rata at reference temperature (\code{TrefR})
 #' @param Q10 Temperature sensitivity of Rd.
 #' @param TrefR Reference temperature for Rd (Celcius).
 #' @param Rdayfrac Ratio of Rd in the light vs. in the dark.
 #' @param EaV,EdVC,delsC Vcmax temperature response parameters
 #' @param EaJ,EdVJ,delsJ Jmax temperature response parameters
+#' @param Km,GammaStar Optionally, provide Michaelis-Menten coefficient for Farquhar model, and Gammastar. If not provided, they are calculated with a built-in function of leaf temperature.
 #' @param Ci Optional, intercellular CO2 concentration (ppm). If not provided, calculated via gs model.
 #' @param Tcorrect If TRUE, corrects input Vcmax and Jmax for actual Tleaf (if FALSE, assumes the provided Vcmax and Jmax are at the Tleaf provided)
 #' @param returnParsOnly If TRUE, returns calculated Vcmax,Jmax,Km and GammaStar based on leaf temperature.
@@ -160,6 +161,9 @@ Photosyn <- function(VPD=1.5,
                      EdVJ = 200000,
                      delsJ = 641.3615,
                      
+                     GammaStar = NULL,
+                     Km = NULL,
+                     
                      Ci = NULL,
                      Tcorrect=TRUE,  
                      returnParsOnly=FALSE,
@@ -196,10 +200,10 @@ Photosyn <- function(VPD=1.5,
   }
   
   # CO2 compensation point in absence of photorespiration
-  GammaStar <- TGammaStar(Tleaf)
+  if(is.null(GammaStar))GammaStar <- TGammaStar(Tleaf)
   
   # Michaelis-Menten coefficient
-  Km <- TKm(Tleaf)
+  if(is.null(Km))Km <- TKm(Tleaf)
   
   #-- Vcmax, Jmax T responses
   if(Tcorrect){
